use std::fmt::{Display, Formatter};
use std::sync::Arc;
use serde_json::Number;

pub mod device;
pub mod emu;

#[derive(Clone, Debug)]
pub struct BroadcastMessage {
    pub id: String,
    pub key: String,
    pub value: Arc<dyn std::any::Any + Send + Sync>,
}

impl Display for BroadcastMessage {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        if let Some(val) = self.value.downcast_ref::<Number>() {
            write!(f, "({}, {})", self.key, val)
        } else if let Some(val) = self.value.downcast_ref::<String>() {
            write!(f, "({}, {})", self.key, val)
        } else {
            write!(f, "({}, {:?})", self.key, self.value.type_id())
        }
    }
}
